
Class cXMLTextWriter
	Private oXML, oCurrentNode, bDocStarted, bDocClosed, mOpenElements
	Public PreserveEmptyNodes

	Private Sub Class_Initialize()
		Set oXML = CreateObject("MSXML2.DomDocument")
		PreserveEmptyNodes = True
		mOpenElements = 0
		bDocStarted = False
		bDocClosed = True
	End Sub
	
	Private Sub Class_Terminate()
		Set oXML = Nothing
	End Sub
	
	Public Function WriteStartDocument(TagName)
		If Not bDocStarted Then
			Set oXML.documentElement = oXML.createElement(TagName)
			Set oCurrentNode = oXML.documentElement
			bDocStarted = True
			bDocClosed = False
		Else
			Err.Raise vbObjectError + 1, "cXMLTextWriter", "The document has already been started"
		End If
	End Function
	
	Public Function WriteEndDocument()
		If mOpenElements = 0 Then
			bDocClosed = True
		Else
			err.Raise vbObjectError + 3, "cXMLTextWriter", "You will have " & mOpenElements & " open element(s). Most Recent: " & oCurrentNode.TagName
		End If
	End Function
	
	Public Function WriteStartElement(TagName)
		If bDocStarted And Not bDocClosed Then
			Dim oNewNode
			Set oNewNode = oXML.createElement(TagName)
			oCurrentNode.appendChild oNewNode
			Set oCurrentNode = oNewNode
			mOpenElements = mOpenElements+1
		Else
			err.Raise vbObjectError + 2, "cXMLTextWriter", "Either the document is not started or has been closed"
		End If
	End Function 
	
	Public Function WriteEndElement()
		If bDocStarted and not bDocClosed then
			Set oCurrentNode=oCurrentNode.parentNode
			mOpenElements=mOpenElements-1
		else
			err.Raise vbObjectError + 2, "cXMLTextWriter", "Either the document is not started or has been closed"
		end if
	End Function
	
	Public Function WriteString(Text)
		If bDocStarted And Not bDocClosed Then
			oCurrentNode.text = Text
		Else
			Err.Raise vbObjectError + 2, "cXMLTextWriter", "Either the document is not started or has been closed"
		End If
	End Function
	
	Public Function WriteAttributeString(AttributeName, AttributeValue)
		If bDocStarted And Not bDocClosed Then
			oCurrentNode.setAttribute AttributeName, AttributeValue
		Else
			Err.Raise vbObjectError + 2, "cXMLTextWriter", "Either the document is not started or has been closed"
		End If
	End Function
	
	Public Function WriteElementString(TagName, NodeText)
		If bDocStarted And Not bDocClosed Then
			If len(NodeText)>0 or PreserveEmptyNodes Then 'only write if there is a value
				Dim oNewNode
				Set oNewNode = oXML.createElement(TagName)
				If len(NodeText) > 0 Then
					oNewNode.Text = NodeText
				End If
				oCurrentNode.appendChild oNewNode
			End If
		Else
			err.Raise vbObjectError + 2, "cXMLTextWriter", "Either the document is not started or has been closed"
		End If
	End Function
	
	Public Property Get xml()
		If bDocClosed Then
			xml = oXML.xml
		Else
			Err.Raise vbObjectError + 4, "cXMLTextWriter", "You must first close the document"
		End If
	End Property
	
	Public Property Get xmlObject()
		If bDocClosed Then
			Set xmlObject = oXML
		Else
			Err.Raise vbObjectError + 4, "cXMLTextWriter", "You must first close the document"
		End If
	End Property

End Class
